/*----------------------

   GATE - Geant4 Application for Tomographic Emission 
   OpenGATE Collaboration 
   OpenGATE Cluster Taskforce

   Jan De Beenhouwer <jan.debeenhouwer@ugent.be> 

   Copyright (C) 2005 ELIS, University of Ghent
   B-9000 Ghent, Belgium  

   Dirk Kruecker extended for seed files 2.6.2005

This software is distributed under the terms 
of the GNU Lesser General  Public Licence (LGPL) 
See GATE/LICENSE.txt for further details 
----------------------*/

#ifndef GateMacfileParser_h
#define GateMacfileParser_h 1
#include "globals.hh"
#include "vector"
#include <iostream> 
#include <sys/types.h>
#include <sys/stat.h>
#include <sstream> 
#include <fstream> 
#include <math.h>

using namespace std;

/*use this class to generate fully resolved macfiles and a splitfile*/

enum {ROOT=0,ASCII=1,INTER=2,SEED=3,ECAT=4,SINO=5,LMF=6,RAW=7,DAQ=8,SIZE=9};

class GateMacfileParser
{
public:

  GateMacfileParser(G4String macfileName,G4int numberOfSplits,G4int numberOfAliases,G4String* aliasesPtr);
  ~GateMacfileParser();
  void SetVerboseLevel(G4int value) { m_verboseLevel = value; };
  G4int GenerateResolvedMacros(G4String directory);
  G4String GetOutputMacDir();
  G4String GetoutputDir(){return outputDir;}; 
  void CleanAbort();
  
 protected:
     
  G4int m_verboseLevel;
  G4String macName;
  G4int nSplits; 
  G4int nAliases;
  G4String* aliases;
  char buffer[256];
  G4String macline;
  G4String localDir;
  G4String outputDir;
  G4double timeStop;
  G4double timeStart;
  G4double virtualStartTime;
  G4double virtualStopTime;
  G4double lambda;
  G4String timeUnit;
  G4String PWD;
  G4String originalRootFileName;
  G4String originalInterFileName;
  G4String originalEcat7FileName;
  G4String originalSinoFileName;
  G4String originalLmf1FileName;
  G4String originalRawFileName;
  G4String originalAsciiFileName;
  G4String Extension;
  G4String outputMacDir;
  void InsertAliases();
  void InsertSubMacros(ofstream& output,G4int splitNumber,ofstream& splitfile);
  void InsertVirtualCommands(ofstream& output,G4int splitNumber,ofstream& splitfile);
  void IgnoreRandomEngineCommand();
  void ExtractLocalDirectory(G4String macfileName);
  G4int GenerateResolvedMacro(G4String outputName,G4int splitNumber,ofstream& splitfile);
  void InsertFileNames(G4int splitNumber,ofstream& splitfile);
  bool IsComment(G4String line);
  void FormatMacline();
  void LookForEnable();
  void CheckOutputPrint();
  void CheckOutput(ofstream&,ofstream&,G4int);
  int enable[SIZE];
  int filenames[SIZE];
  bool* usedAliases;
  bool Braced(G4String origFile);
  void BraceReplace(G4String def, G4String origFile, char* SplitNumberAsString);
  const G4String ExtractFileName(G4String key);
  void RemoveExtension();
  bool TreatOutputStream(G4String key, G4String def, G4String& origFile, char* SplitNumberAsString);
  void AddPWD(G4String key);
  void CalculateTimeSplit(G4int splitNumber);
};
#endif
